# -*- coding: utf-8 -*-
"""
ViewWidget 

Basically a toolbar and key-press manager for a MplCanvas.

Created on Mon May 18 11:45:47 2015

@author: Robert A. McLeod
"""

from PySide import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)
        
import numpy as np
from copy import copy
import functools
# Import static types from MplCanvas
import MplCanvas
import matplotlib.image
from Ui_ViewWidget import Ui_ViewWidget

class ViewWidget(QtGui.QWidget, Ui_ViewWidget, object):
    
    @property 
    def live( self ):
        return self._live
    
    @live.setter
    def live( self, value ):
        value = bool(value)
        if value != self._live:
            self._live = value
            self.tbLive.setChecked(value)
            self.viewCanvas.live = value
            self.tbChangeColormap.setEnabled(value)
            self.tbLogIntensity.setEnabled(value)
            self.tbToggleColorbar.setEnabled(value)
            self.tbToggleHistogramContrast.setEnabled(value)
            self.viewCanvas.updatePlotFunc( self.comboView.currentText() )
            
    def __init__( self, parent=None ):
        """
        QtGui.QWidget.__init__(self)
        self.ui = Ui_ViewWidget()
        
        self.ui.setupUi( self )
        """
        object.__init__(self)
        QtGui.QWidget.__init__(self)
        # Using multiple inheritence setup Ui from QtDesigner
        self.setupUi(self)
        self.parent = parent # This is generally not what I need, what I need is the Automator object
        self.autoParent = None # Set by hand in the Automator.__init__() function
        self.popout = None

        self.viewNumber = 0
        self._live = True
        
        # Re-direct keyPress functions
        self.keyPressEvent = self.grabKey
        # Run through all the widgets and redirect the key-presses to everything BUT the spinboxes
#        widgetlist = self.ui.centralwidget.findChildren( QtGui.QWidget )        
#        print "TO DO: release focus from spinboxes on ENTER key press" 
#        for mywidget in widgetlist:
#            # print "Pause" 
#            if not mywidget.__class__ is QtGui.QDoubleSpinBox:
#                mywidget.keyPressEvent = self.grabKey
        
        # self.
        
        # Connect slots
        self.comboView.currentIndexChanged.connect( self.updatePlotType )

        
        self.tbNextImage.clicked.connect( functools.partial( self.shiftImageIndex, 1 ) )
        self.tbPrevImage.clicked.connect( functools.partial( self.shiftImageIndex, -1 ) )
        
        self.tbShowBoxes.toggled.connect( self.toggleShowBoxes )
        
        # self.leImageIndex.textChanged( self.updateImageIndex )
        self.leImageIndex.textEdited.connect( self.updateImageIndex )
        self.tbToggleColorbar.toggled.connect( self.toggleColorbar )
        self.tbChangeColormap.clicked.connect( self.cycleColormap )
        self.tbLogIntensity.toggled.connect( self.toggleLogInt )
        self.tbPopoutView.toggled.connect( self.popoutViewDialog )
        self.tbLive.toggled.connect( self.toggleLiveView )
        self.sbHistogramCutoff.valueChanged.connect( self.updateHistClim )
        
        # This doesn't work because there's two types of valueChanged sent
        # BUT ONLY WHEN AN IMAGE IS LOADED...
        
        # Has another example with new types
        # http://pyqt.sourceforge.net/Docs/PyQt4/new_style_signals_slots.html
        # Try oldschool connect, deprecated not working?
        # self.connect( self.sbHistogramCutoff, QtCore.SIGNAL('valueChanged(double)'), self.updateHistClim )
        # self.connect
        # This says it's a timer thing due to mouse presses:
        # http://www.qtcentre.org/threads/43078-QSpinBox-Timer-Issue
        self.sbHistogramCutoff.validate = None
    
    
    def grabKey( self, event ):
        # I think some of these key presses aren't being intercepted?
        print( "ViewWidget"+str(self.viewNumber)+"::grabKey : " + str(event.key()) )
        if( event.key() == QtCore.Qt.Key_Down ):
            print( "Down" )  
        elif( event.key() == QtCore.Qt.Key_Up ):
            print( "Up" )  
        elif( event.key() == QtCore.Qt.Key_Left ):
            print( "Left" )  
        elif( event.key() == QtCore.Qt.Key_Right ):
            print( "Right" )  
        elif( event.key() == QtCore.Qt.Key_PageUp ):
            print( "PageUp" )  
        elif( event.key() == QtCore.Qt.Key_PageDown ):
            print( "PageDown" )   
        else:
            return
            

    def toggleLiveView( self ):
        self.live = self.tbLive.isChecked()
        
    def toggleShowBoxes( self ):
        print( "Trying box overlay" )
        self.viewCanvas.plotObj.plotDict['boxMask'] = None
        if self.tbShowBoxes.isChecked():
            # Load from the zorroObj
            try:
                print( "figBoxMask: " + str(self.viewCanvas.zorroObj.files['figBoxMask']) )
            except:
                pass
            if 'figBoxMask' in self.viewCanvas.zorroObj.files:
                self.viewCanvas.plotObj.plotDict['boxMask'] = matplotlib.image.imread( self.viewCanvas.zorroObj.files['figBoxMask'] )
            
        self.viewCanvas.updatePlotFunc( self.comboView.currentText() )
        
    # Decorators to stop multiple events (doesn't work damnit, everything is float)
#    @QtCore.Slot(float)    
#    @QtCore.Slot(str)
    def updateHistClim( self, value ):
        self.viewCanvas.param['cutoff'] = np.power( 10.0, self.sbHistogramCutoff.value())

        self.viewCanvas.updatePlotFunc( self.comboView.currentText() )
        pass
    
    def toggleLogInt( self ):
        # This should call updateHistClim, not sure if I want too
        self.viewCanvas.param['logScale'] = ~self.viewCanvas.param['logScale']
        self.viewCanvas.updatePlotFunc( self.comboView.currentText() )
        
    def popoutViewDialog( self ):
        if self.popout == None:
            self.popout = ViewDialog()
            self.copyDeep( self.popout.view ) # ViewDialog is just a wrapper around ViewWidget 'view'
            self.popout.view.tbPopoutView.setEnabled(False)
        elif self.popout != None:
            self.popout.close()
            self.popout = None
        pass
    
    # Unfortunately the copy library doesn't work nicely with Qt
    def copyDeep( self, thecopy ):
        thecopy.viewNumber = self.viewNumber + 100
        thecopy.parent = self.parent
        thecopy.autoParent = self.autoParent
        thecopy.viewCanvas = copy( self.viewCanvas )

        # No copy of popout
        # Turn events OFF
        thecopy.blockSignals( True )
        print( "BLOCKING SIGNALS" )
        thecopy.tbToggleColorbar.setChecked( self.tbToggleColorbar.isChecked() )
        thecopy.tbLogIntensity.setChecked( self.tbLogIntensity.isChecked() )
        thecopy.tbToggleHistogramContrast.setChecked( self.tbToggleHistogramContrast.isChecked() )
        thecopy.leImageIndex.setText( self.leImageIndex.text() )
        thecopy.sbHistogramCutoff.blockSignals( True )
        thecopy.sbHistogramCutoff.setValue( self.sbHistogramCutoff.value() )
        thecopy.sbHistogramCutoff.blockSignals( False )
        thecopy.blockSignals( False )
        print( "UNBLOCKING SIGNALS" )
        
    def toggleColorbar( self ):
        self.viewCanvas.param['showColorbar'] = ~self.viewCanvas.param['showColorbar']
        self.viewCanvas.updatePlotFunc( self.comboView.currentText() )
        
    def cycleColormap( self ):
        # This is sort of dumb, just have a function inside zorroPlot for this.
        self.viewCanvas.cmap = self.viewCanvas.plotObj.cmaps_cycle.next() 
        self.viewCanvas.plotObj.plotDict['image_cmap'] = self.viewCanvas.cmap
        self.viewCanvas.plotObj.plotDict['graph_cmap'] = self.viewCanvas.cmap
        self.viewCanvas.updatePlotFunc( self.comboView.currentText() )
        
    def shiftImageIndex( self, shift=1 ):
        newIndex = np.int32( self.leImageIndex.text() )
        newIndex += shift
        self.updateImageIndex( imageIndex = newIndex )
        
    def updateImageIndex( self, imageIndex=None ):
        if imageIndex is None:
            imageIndex = np.int( self.leImageIndex.text() )
            
        self.viewCanvas.param['imageIndex'] = imageIndex
        self.refreshCanvas()
        self.leImageIndex.blockSignals( True )
        self.leImageIndex.setText( "%s"%imageIndex ) 
        self.leImageIndex.blockSignals( False )
    
    def updateZorroObj( self, zorroObj = None ):
        self.viewCanvas.updatePlotFunc( self.comboView.currentText(), zorroObj )
            
    def updatePlotType( self, index ):
        # This function is called when you need to update the underlying data of the canvas
        self.viewCanvas.updatePlotFunc( self.comboView.currentText() )
            
        
    def loadConfig( self, config ):
        groupstring = "view" + str(self.viewNumber)
       
        try:  self.comboView.setCurrentIndex( self.comboView.findText( config.get( groupstring, 'plotType' ) ) )
        except: print( "Failed to set plotType for view %d"%self.viewNumber )
        
        try: 
            self.live = config.getboolean( groupstring, 'live')

        except: pass
        try: self.tbToggleColorbar.checked( config.getboolean( groupstring, 'colorBar') )
        except: pass
        try: self.tbToggleHistogramContrast.checked( config.getboolean( groupstring, 'histogramContrast') )
        except: pass
        try: self.tbLogIntensity.checked( config.getboolean( groupstring, 'logIntensity') )
        except: pass
        try: self.sbHistogramCutoff.setValue( config.getint( groupstring, 'histogramCutoff') )
        except: pass
        try: self.tbShowBoxes.checked( config.getboolean( groupstring, 'showBoxes') )
        except: pass
    
    
        try: self.viewCanvas.plotObj.plotDict['image_cmap'] = config.get( groupstring, 'image_cmap' )
        except: pass
        try: self.viewCanvas.plotObj.plotDict['graph_cmap'] = config.get( groupstring, 'graph_cmap' )
        except: pass
        
    def saveConfig( self, config ):
        groupstring = "view" + str(self.viewNumber)
        config.add_section(groupstring)
        config.set( groupstring, 'plotType', self.comboView.currentText() )
        config.set( groupstring, 'live', self.tbLive.isChecked() )
        config.set( groupstring, 'colorBar', self.tbToggleColorbar.isChecked() )
        config.set( groupstring, 'histogramContrast', self.tbToggleHistogramContrast.isChecked() )
        config.set( groupstring, 'logIntensity', self.tbLogIntensity.isChecked() )
        config.set( groupstring, 'histogramCutoff', self.sbHistogramCutoff.value() )
        config.set( groupstring, 'showBoxes', self.tbShowBoxes.isChecked() )
        
        # We can only save some plotDict keys because it might contain a lot of data!
        print( "TODO: save some specific information from plotDict" )
        try: config.set( groupstring, 'image_cmap', self.viewCanvas.plotObj.plotDict['image_cmap'] )
        except: pass
        try: config.set( groupstring, 'graph_cmap', self.viewCanvas.plotObj.plotDict['graph_cmap'] )
        except: pass
        pass

    
    
from Ui_ViewDialog import Ui_ViewDialog
class ViewDialog(QtGui.QDialog, Ui_ViewDialog):
    
    def __init__(self):
        QtGui.QDialog.__init__(self)
        
        self.setupUi(self)
        
        # Set up the user interface from Designer.  
        self.show()
        
        