# -*- coding: utf-8 -*-
"""
Sun Grid Engine (SQE) qsub batch Zorro submission script
Created on Tue Feb  2 16:28:50 2016
@author: Robert A. McLeod

This is currently intended to be run from a <project>/raw/ directory.  Puts results in <project>/align 
and figures in <project>/figs

TODO: this clearly needs to stage the submissions a bit at the start.  IO collision between the processes is 
pretty heavy.
"""

import os, os.path
import glob
import subprocess
import zorro
# import shutil

n_threads = 16
binFactor = 2
queue_name = "all.q@@c-cina"

# Build default zorro cfg 
bin_script = os.path.join( os.path.split(zorro.__file__)[0],  "binDM4.py" )

fileList = glob.glob( "*.dm4" )

process_list = []
for J, filename in enumerate(fileList):
    submitName = "qsub_" + fileList[J] + ".sh"
    submitHeader = """#! /bin/bash
#$ -N %s
#$ -cwd
#$ -S /bin/bash
#$ -p -10
#$ -e %s
#$ -o %s
#$ -j y
#$ -pe mpi_openmp_16 %d

export PATH="/scicore/pfs/c-cina/anaconda2/bin:$PATH"
export PYTHONPATH="$PYTHONPATH"
""" % (submitName, fileList[J] + ".zerr" , fileList[J] + ".zout", n_threads )

    submitCommand  = "python %s"%bin_script + " %s %f %d"% (fileList[J], binFactor, n_threads) + "\n"

    with open( submitName, 'wb' ) as sub:
        sub.write( submitHeader )
        sub.write( submitCommand  )
    print( "Submitting to grid engine : %s"%submitName )
    
    if bool(queue_name):
        process_list.append( subprocess.Popen( "qsub -q " +queue_name+ " "+ submitName, shell=True ) )
    else:
        process_list.append( subprocess.Popen( "qsub " + submitName, shell=True ) ) 

# TODO figure out when we are done-done and delete temporaries?
print( "##### DONE qsub-ing #####" )

