# -*- coding: utf-8 -*-
"""
Sun Grid Engine (SQE) qsub batch Zorro submission script
Created on Tue Feb  2 16:28:50 2016
@author: Robert A. McLeod

This is currently intended to be run from a <project>/raw/ directory.  Puts results in <project>/align 
and figures in <project>/figs

TODO: this clearly needs to stage the submissions a bit at the start.  IO collision between the processes is 
pretty heavy.
"""
import zorro
import os, os.path
import glob
import subprocess
# import shutil

n_threads = 16
queue_name = "all.q@@c-cina"

# Build default zorro cfg 
zorroDefault = zorro.ImageRegistrator()
zorroDefault.maxShift = 60
zorroDefault.shapePadded = [4096,4096]
zorroDefault.fouCrop = [3072,3072]
zorroDefault.shiftMethod = 'lanczos'

zorroDefault.suppressOrigin = True
zorroDefault.triMode = 'diag'
zorroDefault.diagWidth = 6
zorroDefault.weightMode = 'autologistic'
zorroDefault.doDoseFilter = True
zorroDefault.doCompression = False
zorroDefault.doCTF = True
zorroDefault.savePNG = True
zorroDefault.n_threads = n_threads
zorroDefault.plotDict['Transparent'] = False


zorro_script = os.path.join( os.path.split(zorro.__file__)[0],  "zorro.py" )

fileList = glob.glob( "*.dm4" )

process_list = []
logList = [None] * len(fileList)
for J, filename in enumerate(fileList):
    fileFront = os.path.split( os.path.splitext( filename )[0] )[1]
    # Copy the template
    logList[J] = filename + ".log"
    
    zorroDefault.files = {}
    zorroDefault.files['figurePath'] = "../figs"
    zorroDefault.files['config'] = logList[J]
    zorroDefault.files['stack'] = filename
    zorroDefault.files['align'] = os.path.join( "../align", fileFront + "_zorro_movie.mrcs" )
    zorroDefault.files['sum'] = os.path.join( "../align", fileFront + "_zorro.mrc" )

    zorroDefault.saveConfig()
    #shutil.copyfile( template, logname )

for J, logname in enumerate(logList):
    submitName = "qsub_" + fileList[J] + ".sh"
    submitHeader = """#! /bin/bash
#$ -N %s
#$ -cwd
#$ -S /bin/bash
#$ -p -10
#$ -e %s
#$ -o %s
#$ -j y
#$ -pe mpi_openmp_16 %d

export PATH="/scicore/pfs/c-cina/anaconda2/bin:$PATH"
export PYTHONPATH="$PYTHONPATH"
""" % (submitName, fileList[J] + ".zerr" , fileList[J] + ".zout", n_threads )

    submitCommand  = "python %s"%zorro_script + " -c %s"%logList[J] + "\n"

    with open( submitName, 'wb' ) as sub:
        sub.write( submitHeader )
        sub.write( submitCommand  )
    print( "Submitting to grid engine : %s"%submitName )
    
    if bool(queue_name):
        process_list.append( subprocess.Popen( "qsub -q " +queue_name+ " "+ submitName, shell=True ) )
    else:
        process_list.append( subprocess.Popen( "qsub " + submitName, shell=True ) ) 

# TODO figure out when we are done-done and delete temporaries?
print( "##### DONE qsub-ing #####" )

